@@ -313,6 +313,14 @@ sub checkKeys($$)
 #-------------------------------------------------------------------------------
 
 sub genLValueScalarMethods(@)                                                   # Scalar methods
+ {my ($package) = caller;                                                       # Package
+  for(@_)                                                                       # Name each method
+   {my $s = 'sub '.$package.'::'.$_.':lvalue {my $v; $_[0]{"'.$_.'"} //= $v}';
+    eval $s;
+    confess "Unable to create LValue scalar method for: '$_' because\n$@" if $@;
+   }
+ }
+sub genLValueScalarMethodsWithDefaultValues(@)                                  # Scalar methods with default values
  {my ($package) = caller;                                                       # Package
   for(@_)                                                                       # Name each method
    {my $s = 'sub '.$package.'::'.$_.':lvalue {my $v = "'.$_.'"; $_[0]{"'.$_.'"} //= $v}';
@@ -348,6 +356,62 @@ sub indentString($$)
   join "\n", map {$indent.$_} split "\n", (ref($string) ? $$string  : $string)
  }
 
+#-------------------------------------------------------------------------------
+# Trim off white space from from front and end of string
+#-------------------------------------------------------------------------------
+
+sub trim($)
+ {my ($string) = @_;
+  $string =~ s/\A\s+//r =~ s/\s+\Z//r
+ }
+
+#-------------------------------------------------------------------------------
+# Documentation
+#-------------------------------------------------------------------------------
+
+sub extractDocumentation()                                                      # Extract documentation from a perl script - sections are marked with #n, sub name and parameters are on two lines, private methods are marked with ##
+ {my %s;                                                                        # Method names
+  my @d;                                                                        # Documentation
+  my $level = 0; my $off = 1;                                                   # Header levels
+  my @l = split /\n/, readFile($0);                                             # Read this file
+  for my $l(keys @l)
+   {my $L = $l[$l];                                                             # This line
+    my $M = $l[$l+1];                                                           # The next line
+
+    if ($L =~ /\A#(\d)\s+(.*?)\s*(#\s*(.+)\s*)?\Z/)                             # Sections are marked with #n in column 1-2 followed by title followed by optional text
+     {$level = $1;
+      my $h = $level+$off;
+      push @d, "\n=head$h $2" if $level;
+      push @d, "\n$4"         if $level and $4;                                 # Text of section
+     }
+    elsif ($L =~ /\A#/)                                                         # Switch documentation off
+     {$level = 0;
+     }
+    elsif ($level and $L =~ /\A\s*sub\s+(.+?)\s*#\s*(.+?)\s*\Z/)                # Documentation for a method
+     {my ($n, $d) = ($1, $2);                                                   # Name from sub, description from comment
+      next if $d =~ /\A#/;                                                      # Private method if ##
+      $n =~ s/\(.+?\)//;                                                        # Method name
+      my $D = eval '"'.$d.'"'; confess $@ if $@;                                # Evaluate the description
+      my ($p, $c) = $M =~ /\A\s*(.+?)\s*#\s*(.+?)\s*\Z/;                        # Parameters, parameter descriptions from comment
+          $p //= ''; $c //= '';                                                 # No parameters
+      my @p = split /,\s*/, $p =~ s/\A\s*\{my\s*\(//r =~ s/\)\s*=\s*\@_;//r;
+      my @c = split /,\s*/, $c;
+      my $P = join ', ', @p;
+      my $h = $level+$off+1;
+      my $N = "$n($P)";                                                         # Method
+      $s{$N}++;                                                                 # Index method names
+      push @d, "\n=head$h $N\n\n $D\n";                                         # Method description
+      push @d, indentString(formatTable([[qw(Parameter Description)], map{[$p[$_], $c[$_]]} keys @p]), '  ')
+        if $p and $c and $c !~ /\A#/;
+     }
+   }
+  push @d, "\n\n=head$off Index of methods by name\n";                          # Index
+  push @d, "L</$_>" for sort keys %s;                                           # Alphabetic listing of methods
+  push @d, "\n";
+  writeFile("doc.data", join "\n", @d);                                         # Write file
+ }
+# Need to apply this method to this file
+
 #-------------------------------------------------------------------------------
 # Examples
 #-------------------------------------------------------------------------------
@@ -368,8 +432,6 @@ if (0 and !caller)
 
 eval join('', <Data::Table::Text::DATA>) || die $@ unless caller();
 
-#say STDERR formatTable(\%Data::Table::Text::);
-
 #-------------------------------------------------------------------------------
 # Export
 #-------------------------------------------------------------------------------
@@ -379,17 +441,20 @@ require Exporter;
 
 use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);
 
+#say STDERR formatTable(\%Data::Table::Text::);
+
 @ISA          = qw(Exporter);
 @EXPORT       = qw(formatTable);
-@EXPORT_OK    = qw(checkKeys dateStamp dateTimeStamp fileList fileSize
-genLValueArrayMethods genLValueHashMethods genLValueScalarMethods
+@EXPORT_OK    = qw(checkKeys dateStamp dateTimeStamp
+extractDocumentation fileList fileSize
+genLValueArrayMethods genLValueHashMethods genLValueScalarMethods genLValueScalarMethodsWithDefaultValues
 indentString
 loadArrayArrayFromLines loadArrayFromLines  loadHashArrayFromLines loadHashFromLines
 makePath
-readBinaryFile readFile timeStamp writeBinaryFile writeFile);
+readBinaryFile readFile timeStamp trim writeBinaryFile writeFile);
 %EXPORT_TAGS  = (all=>[@EXPORT, @EXPORT_OK]);
 
-$VERSION = '1.006'; # Thursday 15 December 2016
+$VERSION = '1.007'; # Sunday 18 December 2016
 
 1;
 
@@ -499,7 +564,7 @@ modified under the same terms as Perl itself.
 =cut
 
 __DATA__
-use Test::More tests => 29;
+use Test::More tests => 30;
 if (1)                                                                          # Unicode
  {use utf8;
   my $z = "𝝰𝝱𝝲";
@@ -600,9 +665,15 @@ if (1)
  {package Scalars;
   my $a = bless{};
   Data::Table::Text::genLValueScalarMethods(qw(aa bb cc));
+  $a->aa = 'aa';
+  Test::More::ok $a->aa eq 'aa';
+ }
+
+if (1)                                                                          # SM
+ {package ScalarsWithDefaults;
+  my $a = bless{};
+  Data::Table::Text::genLValueScalarMethodsWithDefaultValues(qw(aa bb cc));
   Test::More::ok $a->aa eq 'aa';
-  $a->aa = 'bb';
-  Test::More::ok $a->aa eq 'bb';
  }
 
 if (1)                                                                          # AM
@@ -626,3 +697,5 @@ if (1)
   my $s = indentString(formatTable($d), '  ') =~ s/\n/|/gr;
   ok $s eq '     .  aa   bb   cc   |  1  1  A    B    C    |  2  2  AA   BB   CC   |  3  3  AAA  BBB  CCC  |  4  4    1   22  333  ';
  }
+
+ ok trim(" a b ") eq join ' ', qw(a b);                                         # Trim
